# Torch EMA

Exponential Moving Average (EMA) is a type of moving average, which applies more weight to the most recent data points than to those that happened earlier in the past.
So to say it focuses more on the most recent data than to older ones. 

## Requirements

```
python
torch
```

## Example

```python
import torch
from torch_ema import TorchEMA as EMA

model = torch.nn.Linear(128, 128)

ema = EMA(
    model=model,
    beta = 0.9999,
    update_after_step = 100,
    update_every = 10,
)

with torch.no_grad():
    model.weight.copy_(torch.randn_like(model.weight))
    model.bias.copy_(torch.randn_like(model.bias))

ema.update()

rnd_data = torch.randn(1, 128)

output     = model(rnd_data)
ema_output = ema(rnd_data)
```