from typing import Any, Optional, Union

import torch
from torch import nn

FLOAT_TYPES = [torch.float64, torch.float32, torch.float16, torch.bfloat16]

def exists(val: Any) -> bool:
    """
    Returns whether the value exists or not.

    Returns:
        True if value is not None.
    """
    return val is not None

def is_float_dtype(dtype: Any) -> bool:
    """
    Returns whether the given type dtype is of float.

    Returns:
        True if dtype is of type float, false otherwise.
    """
    return any([dtype == float_dtype for float_dtype in FLOAT_TYPES])

def clamp(
    value: Union[int, float],
    min_value: Optional[Union[int, float]] = None,
    max_value: Optional[Union[int, float]] = None
) -> Union[int, float]:
    """
    Clamps the provided value.

    Args:
        value:
            The value to clamp.
        min_value:
            The minimum value.
        max_value:
            The maximum value.

    Returns:
        Value clamped between min_value and max_value
    """
    assert exists(min_value) or exists(max_value)
    if exists(min_value):
        value = max(value, min_value)
    if exists(max_value):
        value = min(value, max_value)

    return value